      program pifmapdel
      implicit none
      
      include '../include/pifhead.inc'
      type(pifhead_t) outhead, inhead

      character outmap*64, inmap*64, pdbfile*64, a1*64, a2*64
      character(len=1) rw(2)
      integer   i, j, k, ncol, nrow, nsec
      integer   ix1, ix2, iy1, iy2, iz1, iz2, rx, ry, r
      real      border, xx, yy, zz, pixelsize
      real*4, allocatable :: map3d(:,:,:)

      write(*,*)
      write(*,*) 'Mask a map based on a pdb file '
      write(*,*)
      write(*,*) ' Author: W. Zhang, June 19, 2002 '
      write(*,*) ' Modified: RS Sinkovits, August 10, 2010 '
      write(*,*)

C     --- Read parameters from stdin
 10   write(*,*) 'border (in pixel float): '
      read (*,*,err=10) border
 11   write(*,*) 'input PIF map: '
      read (*,'(a)', err=11) inmap
 12   write(6,*) 'output PIF map: '
      read (5,'(a)',err=12) outmap
 13   write(*,*) 'pixel size (Ang): '
      read (*, * ,err=13) pixelsize
 14   write(*,*) 'pdb filename: '
      read (*, '(a)',err=14) pdbfile

C     --- Read input map header and data
      inhead%lunum = 7      ! unit for input image file
      rw(1) = 'r'
      rw(2) = achar(0)
      call pif_open(inmap, inhead%lunum, rw)
      call pif_read_gh(inhead)
      ncol = inhead%numcols
      nrow = inhead%numrows
      nsec = inhead%numsecs
      inhead%imgnum = 1
      call pif_read_dh(inhead)
      allocate(map3d(ncol,nrow,nsec))
      call pif_read_wmapi2(inhead, map3d, ncol, nrow, nsec)
      call pif_close(inhead%lunum)

c     --- Open output map / fill and write header
      outhead%lunum = 11
      rw(1) = 'w'
      rw(2) = '+'
      call pif_open(outmap, outhead%lunum, rw)
      outhead = inhead          ! Structure assignment

      outhead%binfactor  =   1
      outhead%nimg       =   1
      outhead%mode       =  20
      outhead%numcols     = ncol ! unbinned size
      outhead%numrows     = nrow ! unbinned size
      outhead%numsecs     = nsec ! unbinned size

      call pif_write_gh(outhead) 
      outhead%imgnum = 1
      call pif_write_dh(outhead)
      
C     --- Apply PDB coordinates to map
      open (unit=2, file=pdbfile, err=996)
 19   read(2, '(a30,3f8.3,a12)',end=900, err=996) a1,
     $     xx, yy, zz, a2
      
c     Convert coordinates to pixel numbers
      ix1 = xx/pixelsize - border + (ncol+1)/2
      ix2 = xx/pixelsize + border + (ncol+1)/2 + 0.5
      iy1 = yy/pixelsize - border + (nrow+1)/2
      iy2 = yy/pixelsize + border + (nrow+1)/2 + 0.5
      iz1 = zz/pixelsize - border + (nsec+1)/2
      iz2 = zz/pixelsize + border + (nsec+1)/2 + 0.5

      if(ix1 < 1)    ix1 = 1
      if(iy1 < 1)    iy1 = 1 
      if(iz1 < 1)    iz1 = 1
      if(ix2 > ncol) ix2 = ncol
      if(iy2 > nrow) iy2 = nrow
      if(iz2 > nsec) iz2 = nsec

      do i = ix1, ix2
         rx = (i - (xx/pixelsize+(ncol+1)/2))**2
         do j = iy1, iy2
            ry = (j - (yy/pixelsize+(nrow+1)/2))**2
            do k = iz1, iz2
               r=sqrt(rx + ry + (k - (zz/pixelsize+(nsec+1)/2))**2)
               if (r <= border) then
                  map3d(i, j, k) = 0.0
               endif
            enddo
         enddo
      enddo
      goto 19

C     --- Done with PDB - write map and wrap things up
 900  call pif_write_map_sections(outhead, map3d, outhead%mode,
     $     ncol, nrow, nsec, 0, nsec)

 996  write(*,*) '*********************************'
      write(*,*) '* finish cut map                *'
      write(*,*) '*********************************'
      close(unit=2)
      call pif_close(outhead%lunum)
      deallocate(map3d)

      end program pifmapdel
