// FIXPIF - application for editing PIF header files
//
//    Rob Ashmore
//    Biology Department
//    Purdue University
//    Sept 1998

#define VERSION "2.00_20021127"

#include "../include/common.h"
#include "fixpif.h"
#include "../libR/libUtil.h"
#include <math.h>

// ---------------------------------------------------------------------
// OS transfer point.  The main routine does all the one-time setup and
// then calls XtAppMainLoop
// ---------------------------------------------------------------------

int main(int argc, char **argv)
{
  char pathenv[1024];
  int i, j;
  String fallbacks[] =
    { "*fontList:-dec-terminal-bold-r-normal--14-140-75-75-c-80-iso8859-1=tag",
      NULL
    };
  
  banner("fixpif");

  // Init
  currentHeader = 0;
  numImages     = 1;
  fileIsOpen    = FALSE;
  dirty         = FALSE;
  
  fileName.addr = (char *)calloc(2048,sizeof(char));
  fileName.length = 2048;
  
  // Initialize MRM
  MrmInitialize();                 
  
  XtToolkitInitialize();
  app_context = XtCreateApplicationContext();
  
  XtAppSetFallbackResources(app_context, fallbacks);
  
  // Finally, open the display
  dsply = XtOpenDisplay(app_context, NULL, "fixpif",
			"fixpif", NULL, 0, &argc, argv);
  
  // Add our remapping of keys to widgets
  XtAppAddActions(app_context,rwa_action_table, 1); 
  
  // Explicitly call this class "fixpif"

  toplevel_widget = XtVaAppCreateShell(argv[0], "fixpif",
				       topLevelShellWidgetClass, 
				       dsply, NULL);
  
  // Construct ':' delimited path(s) to be searched for uid file
  memset(pathenv, '\0', 1024);
  strcpy(pathenv, argv[0]);
  for (i=strlen(pathenv)-1; i>=0; i--) {
    if(pathenv[i] == '/') {
      pathenv[i] = ':';
      break;
    }
    else
      pathenv[i] = '\0';
  }
  strcat(pathenv, getenv("PATH"));

  db_filename_vec[0] = (char *)malloc(sizeof(char)*512);
  memset(db_filename_vec[0], '\0', 512);
  j = 0;

  // Check each element of path for uid file
  for(i=0; i<strlen(pathenv); i++) {
    if(pathenv[i] != ':') {
      db_filename_vec[0][j] = pathenv[i];
      j++;
    } 
    else {
      strcat(db_filename_vec[0], "/fixpif.uid");
      if( (fIn = fopen(db_filename_vec[0],"r")) != NULL ) {
	fclose(fIn);
	break;
      } 
      else {
	j = 0;
	memset(db_filename_vec[0], '\0', 512);
      }
    }
  }

  if(j == 0) {
    printf("Unable to find fixpif.uid in PATH\n");
    printf("Make sure that the uid file is in same directory as fixpif\n");
    exit(0);
  }

  printf("Full path to UID file: [%s]\n",db_filename_vec[0]);
  
  // Open UID file in hierarchy
  db_filename_num = (sizeof db_filename_vec / sizeof db_filename_vec [0]);
  if (MrmOpenHierarchy(db_filename_num, // Number of files
		       db_filename_vec, // Array of file names
		       NULL,            // Default OS extenstion
		       &s_MrmHierarchy) // Pointer to returned MRM ID
      !=MrmSUCCESS)
    s_error("can't open hierarchy");
  
  // Register the items MRM needs to bind for us
  MrmRegisterNames(reglist, reglist_num);
  
  // Go get the main part of the application
  if (MrmFetchWidget(s_MrmHierarchy, "S_MAIN_WINDOW", toplevel_widget,
		     &main_window_widget, &dummy_class) != MrmSUCCESS)
    s_error("can't fetch main window");
  
  // Go get the infoScreen part of the application
  if (MrmFetchWidget(s_MrmHierarchy, "infoMessageBox", toplevel_widget,
		     &infoBox, &dummy_class) != MrmSUCCESS)
    s_error("can't fetch info window");
  XtUnmanageChild(XmMessageBoxGetChild(infoBox,XmDIALOG_HELP_BUTTON));

  // Manage the main part and realize everything. The interface comes
  // up on the display now. Manage all of the screens now so we can
  // alter what we need to in XLibInitialize() - Unmanage the unneeded
  // ones thereafter.

  XtManageChild(main_window_widget);
  XFlush(dsply);
  XtUnmanageChild(imageForm);

  XlibInitialize();

  // Sit around forever waiting to process X-events. We never leave
  // XtAppMainLoop. From here on, we only execute our callback routines.
  XtAppMainLoop(app_context);

  return(0);

}



void XlibInitialize()
{
  // function name: XlibInitialize
  // Initialize for Xlib calls.

  // Enable save unders and backing store
  if(XDoesSaveUnders(XScreenOfDisplay(dsply, DefaultScreen(dsply))) &&
     XDoesBackingStore(XScreenOfDisplay(dsply, DefaultScreen(dsply)))) {
    attrs.save_under = TRUE;
    attrs.backing_store = WhenMapped;
    printf("Enabling 'Backing Store' and 'SaveUnder' features\n");
  }

  XFlush (dsply);
  
} // end function XlibInitialize



void s_error(char *problem_string)
{
  // function name: s_error
  // All errors are fatal.

  printf("%s\n", problem_string);
  exit(0);

} // end function s_error



void raiseIt(Widget w)
{
  // function name: raiseIt

  // Do everything necessary to raise a window to the foreground
  // on the display.

  XtManageChild(w);
  XtPopup(XtParent(w), XtGrabNone);

  XmProcessTraversal(w, XmTRAVERSE_CURRENT); 
  XtRealizeWidget(w);  
  XRaiseWindow(dsply, XtWindow(XtParent(w)));
  XFlush(dsply);

} // end function raiseIt



void handle_cr(Widget widget)
{
  // function name: handle_cr
  // This isn't really a callback but it is called by a callback. This
  // routine changes the translation table interpretations for the
  // simple text widgets so it knows what to do when it sees a
  // carriage return.

  XtTranslations parsed_t_table;

  if(widget==headNumTextW) {
    // Do translations for study text widget
    // Modify this widgets translation table
    parsed_t_table = XtParseTranslationTable(rwa_translation_table);
    XtOverrideTranslations(widget, parsed_t_table);
  }
  
} // end function handle_cr



void stuffGlobal()
{
  // function name: stuffGlobal
  
  XtUnmanageChild(imageForm);
  XtManageChild(globalForm);
  
  pif_read_gh(&pifHeader);
  numImages     = pifHeader.nimg;
  currentHeader = 0;

  XmTextSetString(headNumTextW, "Glob");
  TextSetInt(glbHeadTextW[0], pifHeader.nimg);

  sprintf(outStr,"%E", pifHeader.floatScaleFactor);
  XmTextSetString(glbHeadTextW[1], outStr);

  sprintf(outStr,"%E", pifHeader.recipFloatScaleFactor);
  XmTextSetString(glbHeadTextW[2], outStr);

  TextSetInt(glbHeadTextW[3],      pifHeader.htype);
  TextSetInt(glbHeadTextW[4],      pifHeader.numCols);
  TextSetInt(glbHeadTextW[5],      pifHeader.numRows);
  TextSetInt(glbHeadTextW[6],      pifHeader.numSecs);
  TextSetInt(glbHeadTextW[7],      pifHeader.mode);
  TextSetInt(glbHeadTextW[8],      pifHeader.endianNess);
  XmTextSetString(glbHeadTextW[9], pifHeader.versionStr);
  TextSetInt(glbHeadTextW[10],     pifHeader.even);
  TextSetInt(glbHeadTextW[11],     pifHeader.mrcX);
  TextSetInt(glbHeadTextW[12],     pifHeader.mrcY);
  TextSetInt(glbHeadTextW[13],     pifHeader.mrcZ);
  TextSetFlt(glbHeadTextW[14],     pifHeader.int4scaleF);

  XFlush(dsply);

  dirty = FALSE;

} // end function stuffGlobal



void stuffImage(int imgNum)
{
  // function name: stuffImage
  
  XtManageChild(imageForm);
  XtUnmanageChild(globalForm);
  
  pifHeader.imgNum = imgNum;
  pif_read_dh(&pifHeader);

  TextSetInt(headNumTextW, imgNum);

  TextSetInt(imgHeadTextW[0],  pifHeader.imgNum);
  TextSetInt(imgHeadTextW[1],  pifHeader.mode);
  TextSetFlt(imgHeadTextW[2],  pifHeader.img_bkg);
  TextSetInt(imgHeadTextW[3],  pifHeader.datai4_min);
  TextSetInt(imgHeadTextW[4],  pifHeader.datai4_max);
  TextSetFlt(imgHeadTextW[5],  pifHeader.datar4_min);
  TextSetFlt(imgHeadTextW[6],  pifHeader.datar4_max);
  TextSetFlt(imgHeadTextW[7],  pifHeader.datar4_avg);
  TextSetFlt(imgHeadTextW[8],  pifHeader.datar4_std);
  TextSetInt(imgHeadTextW[9],  pifHeader.numCols);
  TextSetInt(imgHeadTextW[10], pifHeader.numRows);
  TextSetInt(imgHeadTextW[11], pifHeader.numSecs);
  XmTextSetString(imgHeadTextW[12], pifHeader.descriptionStr);
  TextSetFlt(imgHeadTextW[13], pifHeader.aoverb);
  TextSetFlt(imgHeadTextW[14], pifHeader.map_abang);
  TextSetFlt(imgHeadTextW[15], pifHeader.a_cell);
  TextSetFlt(imgHeadTextW[16], pifHeader.b_cell);
  TextSetFlt(imgHeadTextW[17], pifHeader.c_cell);
  TextSetFlt(imgHeadTextW[18], pifHeader.alp);
  TextSetFlt(imgHeadTextW[19], pifHeader.bet);
  TextSetFlt(imgHeadTextW[20], pifHeader.gam);
  TextSetFlt(imgHeadTextW[21], pifHeader.dela);
  TextSetFlt(imgHeadTextW[22], pifHeader.delb);
  TextSetFlt(imgHeadTextW[23], pifHeader.delc);
  TextSetFlt(imgHeadTextW[24], pifHeader.a_star);
  TextSetFlt(imgHeadTextW[25], pifHeader.b_star);
  TextSetFlt(imgHeadTextW[26], pifHeader.c_star);
  TextSetFlt(imgHeadTextW[27], pifHeader.alp_star);
  TextSetFlt(imgHeadTextW[28], pifHeader.bet_star);
  TextSetFlt(imgHeadTextW[29], pifHeader.gam_star);
  TextSetFlt(imgHeadTextW[30], pifHeader.pixelSize);
  TextSetInt(imgHeadTextW[31], pifHeader.t_matrix[0]);
  TextSetInt(imgHeadTextW[32], pifHeader.t_matrix[1]);
  TextSetInt(imgHeadTextW[33], pifHeader.t_matrix[2]);
  TextSetInt(imgHeadTextW[34], pifHeader.t_matrix[3]);
  TextSetInt(imgHeadTextW[35], pifHeader.t_matrix[4]);
  TextSetInt(imgHeadTextW[36], pifHeader.t_matrix[5]);
  TextSetInt(imgHeadTextW[37], pifHeader.packRadius);
  TextSetInt(imgHeadTextW[38], pifHeader.binFactor);
  TextSetFlt(imgHeadTextW[39], pifHeader.dthe);
  TextSetFlt(imgHeadTextW[40], pifHeader.dphi_90);
  TextSetInt(imgHeadTextW[41], pifHeader.symmetry);
  TextSetInt(imgHeadTextW[42], pifHeader.pixSizUnits);
  TextSetFlt(imgHeadTextW[43], pifHeader.res_out);
  TextSetInt(imgHeadTextW[44], pifHeader.ctfMode);
  TextSetFlt(imgHeadTextW[45], pifHeader.tempFactor);
  TextSetFlt(imgHeadTextW[46], pifHeader.wiener);
  TextSetInt(imgHeadTextW[47], pifHeader.borderType);
  TextSetInt(imgHeadTextW[48], pifHeader.borderWidth);
  TextSetInt(imgHeadTextW[49], pifHeader.rad_lo);
  TextSetInt(imgHeadTextW[50], pifHeader.rad_hi);
  TextSetInt(imgHeadTextW[51], pifHeader.transX);
  TextSetInt(imgHeadTextW[52], pifHeader.transY);
  TextSetInt(imgHeadTextW[53], pifHeader.transZ);
  TextSetInt(imgHeadTextW[54], pifHeader.isTrunc);
  TextSetInt(imgHeadTextW[55], pifHeader.origX);
  TextSetInt(imgHeadTextW[56], pifHeader.origY);
  TextSetInt(imgHeadTextW[57], pifHeader.origZ);
  TextSetFlt(imgHeadTextW[58], pifHeader.pifVersion);

  XFlush(dsply);
  dirty = FALSE;

} // end function stuffImage



void getImage(int imgNum)
{
  // function name: getImage

  char *thisStr;
  int i;

  pifHeader.mode       = strtod(XmTextGetString(imgHeadTextW[1]), 0);
  pifHeader.img_bkg    = strtod(XmTextGetString(imgHeadTextW[2]), 0);
  pifHeader.datai4_min = strtod(XmTextGetString(imgHeadTextW[3]), 0);
  pifHeader.datai4_max = strtod(XmTextGetString(imgHeadTextW[4]), 0);
  pifHeader.datar4_min = strtod(XmTextGetString(imgHeadTextW[5]), 0);
  pifHeader.datar4_max = strtod(XmTextGetString(imgHeadTextW[6]), 0);
  pifHeader.datar4_avg = strtod(XmTextGetString(imgHeadTextW[7]), 0);
  pifHeader.datar4_std = strtod(XmTextGetString(imgHeadTextW[8]), 0);
  pifHeader.numCols    = strtod(XmTextGetString(imgHeadTextW[9]), 0);
  pifHeader.numRows    = strtod(XmTextGetString(imgHeadTextW[10]), 0);
  pifHeader.numSecs    = strtod(XmTextGetString(imgHeadTextW[11]), 0);

  thisStr= XmTextGetString(imgHeadTextW[12]);
  thisStr[79]= (char)'\0';
  sprintf(pifHeader.descriptionStr,thisStr);
  XtFree(thisStr);

  pifHeader.aoverb    = strtod(XmTextGetString(imgHeadTextW[13]), 0);
  pifHeader.map_abang = strtod(XmTextGetString(imgHeadTextW[14]), 0);
  pifHeader.a_cell    = strtod(XmTextGetString(imgHeadTextW[15]), 0);
  pifHeader.b_cell    = strtod(XmTextGetString(imgHeadTextW[16]), 0);
  pifHeader.c_cell    = strtod(XmTextGetString(imgHeadTextW[17]), 0);
  pifHeader.alp       = strtod(XmTextGetString(imgHeadTextW[18]), 0);
  pifHeader.bet       = strtod(XmTextGetString(imgHeadTextW[19]), 0);
  pifHeader.gam       = strtod(XmTextGetString(imgHeadTextW[20]), 0);
  pifHeader.dela      = strtod(XmTextGetString(imgHeadTextW[21]), 0);
  pifHeader.delb      = strtod(XmTextGetString(imgHeadTextW[22]), 0);
  pifHeader.delc      = strtod(XmTextGetString(imgHeadTextW[23]), 0);
  pifHeader.a_star    = strtod(XmTextGetString(imgHeadTextW[24]), 0);
  pifHeader.b_star    = strtod(XmTextGetString(imgHeadTextW[25]), 0);
  pifHeader.c_star    = strtod(XmTextGetString(imgHeadTextW[26]), 0);
  pifHeader.alp_star  = strtod(XmTextGetString(imgHeadTextW[27]), 0);
  pifHeader.bet_star  = strtod(XmTextGetString(imgHeadTextW[28]), 0);
  pifHeader.gam_star  = strtod(XmTextGetString(imgHeadTextW[29]), 0);
  pifHeader.pixelSize = strtod(XmTextGetString(imgHeadTextW[30]), 0);

  for(i=0; i<6; i++)
    pifHeader.t_matrix[i] = strtod(XmTextGetString(imgHeadTextW[(31+i)]), 0);

  pifHeader.packRadius  = strtod(XmTextGetString(imgHeadTextW[37]), 0);
  pifHeader.binFactor   = strtod(XmTextGetString(imgHeadTextW[38]), 0);
  pifHeader.dthe        = strtod(XmTextGetString(imgHeadTextW[39]), 0);
  pifHeader.dphi_90     = strtod(XmTextGetString(imgHeadTextW[40]), 0);
  pifHeader.symmetry    = strtod(XmTextGetString(imgHeadTextW[41]), 0);
  pifHeader.pixSizUnits = strtod(XmTextGetString(imgHeadTextW[42]), 0);
  pifHeader.res_out     = strtod(XmTextGetString(imgHeadTextW[43]), 0);
  pifHeader.ctfMode     = strtod(XmTextGetString(imgHeadTextW[44]), 0);
  pifHeader.tempFactor  = strtod(XmTextGetString(imgHeadTextW[45]), 0);
  pifHeader.wiener      = strtod(XmTextGetString(imgHeadTextW[46]), 0);
  pifHeader.borderType  = strtod(XmTextGetString(imgHeadTextW[47]), 0);
  pifHeader.borderWidth = strtod(XmTextGetString(imgHeadTextW[48]), 0);
  pifHeader.rad_lo      = strtod(XmTextGetString(imgHeadTextW[49]), 0);
  pifHeader.rad_hi      = strtod(XmTextGetString(imgHeadTextW[50]), 0);
  pifHeader.transX      = strtod(XmTextGetString(imgHeadTextW[51]), 0);
  pifHeader.transY      = strtod(XmTextGetString(imgHeadTextW[52]), 0);
  pifHeader.transZ      = strtod(XmTextGetString(imgHeadTextW[53]), 0);
  pifHeader.isTrunc     = strtod(XmTextGetString(imgHeadTextW[54]), 0);
  pifHeader.origX       = strtod(XmTextGetString(imgHeadTextW[55]), 0);
  pifHeader.origY       = strtod(XmTextGetString(imgHeadTextW[56]), 0);
  pifHeader.origZ       = strtod(XmTextGetString(imgHeadTextW[57]), 0);
  pifHeader.pifVersion  = strtod(XmTextGetString(imgHeadTextW[58]), 0);

  XFlush(dsply);

} // end function getImage



void getGlobal(int imgNum)
{
  // function name: getGlobal

  char *thisStr;

  pifHeader.nimg                 = strtod(XmTextGetString(glbHeadTextW[0]), 0);
  pifHeader.floatScaleFactor     = strtod(XmTextGetString(glbHeadTextW[1]), 0);
  pifHeader.recipFloatScaleFactor= strtod(XmTextGetString(glbHeadTextW[2]), 0);
  pifHeader.htype                = strtod(XmTextGetString(glbHeadTextW[3]), 0);
  pifHeader.numCols              = strtod(XmTextGetString(glbHeadTextW[4]), 0);
  pifHeader.numRows              = strtod(XmTextGetString(glbHeadTextW[5]), 0);
  pifHeader.numSecs              = strtod(XmTextGetString(glbHeadTextW[6]), 0);
  pifHeader.mode                 = strtod(XmTextGetString(glbHeadTextW[7]), 0);
  pifHeader.endianNess           = strtod(XmTextGetString(glbHeadTextW[8]), 0);

  thisStr= XmTextGetString(glbHeadTextW[9]);
  thisStr[79]= '\0';
  sprintf(pifHeader.versionStr,thisStr);
  XtFree(thisStr);

  pifHeader.even       = strtod(XmTextGetString(glbHeadTextW[10]), 0);
  pifHeader.mrcX       = strtod(XmTextGetString(glbHeadTextW[11]), 0);
  pifHeader.mrcY       = strtod(XmTextGetString(glbHeadTextW[12]), 0);
  pifHeader.mrcZ       = strtod(XmTextGetString(glbHeadTextW[13]), 0);
  pifHeader.int4scaleF = strtod(XmTextGetString(glbHeadTextW[14]), 0);

  XFlush(dsply);

} // end function getGlobal



void read_pif()
{
  // function name: read_pif
  // function name: Read image(s) in pif format.

  int i=4;
  fString_t rWrite;

  if(fileIsOpen)
    pif_close(&i); // Close it before we go on

  pifHeader.luNum = i;
  rWrite.addr = malloc(sizeof(char)*4);
  rWrite.length = 1;
  sprintf(rWrite.addr,"r+");
  pif_open(fileName.addr, &i, rWrite.addr);

  fileIsOpen = TRUE;

  // Now stuff text widgets with what you've found
  stuffGlobal();

} // end function read_pif



void gotFilename(Widget w, char *client_data,
		 XmFileSelectionBoxCallbackStruct *callback_data)
{
  // function name: gotFilename
  // Get the filename they specified in file selection widget. Branch
  // according to what operation they want performed on this filename.
  
  XmString simpleString;

  // Pop down the file selection widget
  XtUnmanageChild(fileSel_widget);
  XFlush(dsply);
  
  // Get the filename
  if(!XmStringGetLtoR(callback_data->value, XmSTRING_DEFAULT_CHARSET, 
		      &(fileName.addr)))
    printf("Error extracting filename.\n");
  
  // Read the file
  read_pif();
  
  // Give feedback on main window title bar
  sprintf(outStr,"FixPIF: [%s] - %d images total.",fileName.addr,
	  pifHeader.nimg);
  simpleString = XmStringCreateLocalized(outStr);
  XtSetArg(arglst[0], XmNdialogTitle, simpleString );
  XtSetArg(arglst[1], XmNrecomputeSize, FALSE );
  XtSetValues(main_window_widget, arglst, 2); 
  XFlush(dsply);
  XmStringFree(simpleString); 
  XFlush(dsply);
  
} // end function gotFilename



void cancel_fileSel(Widget widget, char *tag,
		    XmFileSelectionBoxCallbackStruct *callback_data)
{
  // function name: cancel_fileSel
  // They want to abort the selection of a file name.

  XtUnmanageChild(fileSel_widget);

} // end function cancel_fileSel



void imgNumActivate(Widget widget, int *tag,
		    XmAnyCallbackStruct *callback_data)
{
  // function name: imgNumActivate
  // When the "ImgNum" button is pressed redisplay the window
  // using the new imgNum.

  char *thisStr;

  if(fileIsOpen) {

    if(dirty) { // Check to see if we need to write first
      if(askThem("Do you want to save your changes to this header?",
		 XmDIALOG_QUESTION))
	write_activate(widget, tag, callback_data);
    }

    thisStr= XmTextGetString(headNumTextW);
    currentHeader = strtod(thisStr,0);
    XtFree(thisStr);
    
    // Clip it accordingly
    currentHeader = MIN(currentHeader, numImages);
    currentHeader = MAX(currentHeader, 0);
    
    sprintf(outStr,"%d",currentHeader);
    XmTextSetString(headNumTextW, outStr);
    
    if(currentHeader==0)
      stuffGlobal();
    else 
      stuffImage(currentHeader);
    
  }
  else
    askThem("You must open a file first.", XmDIALOG_INFORMATION);
  
} // end function imgNumActivate



void text_changed(Widget widget, int *tag,
		  XmAnyCallbackStruct *callback_data)
{
  // function name: text_changed

  dirty = TRUE;

} // end function text_changed



void text_create(Widget widget, int *tag,
		 XmAnyCallbackStruct *callback_data)
{
  // function name: text_create
  // Create all of the text widgets needed for each PIF header attribute.

  char numStr[20];
  int i, totX;
  Arg arglist[9];
  XFontStruct *font;
  XmFontList font_list;
  XmStringCharSet cs= "ISOLatin1";

  // Load all of these guys with proportional "fixed" font
  font = XLoadQueryFont(dsply, "fixed");
  font_list = XmStringCreateFontList(font, cs); 
  
  totX= 0;
  
  if(*tag==HEADNUM) { // Header number text widget
    headNumTextW = widget;
    handle_cr(widget);
  }
  else if(*tag==GLOBAL) { // global form
    globalForm = widget;
    
    for(i=0; i<MAXGLBFIELDS; i++) { // Do all fields
      XtSetArg(arglist[0], XmNx, 130);
      XtSetArg(arglist[1], XmNy, (50 + i*30));
      XtSetArg(arglist[2], XmNcolumns, 50);
      XtSetArg(arglist[3], XmNfontList, font_list);      
      XtSetArg(arglist[4], XmNeditable, FALSE);      
      sprintf(numStr,"Number%d",i);
      
      glbHeadTextW[i] = XmCreateText(widget, numStr, arglist, 3);
      XtAddCallback(glbHeadTextW[i], XmNvalueChangedCallback, 
		    (XtCallbackProc) text_changed, NULL);
      XtManageChild(glbHeadTextW[i]);
      
      XtSetArg(arglist[0], XmNx, 5);
      XtSetArg(arglist[1], XmNy, (50 + (i*30)));
      XtSetArg(arglist[2], XmNalignment, XmALIGNMENT_BEGINNING);
      XtSetArg(arglist[3], XmNlabelString, 
	       XmStringLtoRCreate(glbFields[i], cs));
      
      XtManageChild(XmCreateLabel(widget, "", arglist, 4));  
    }
  }
  else if (*tag==IMAGE) { // Image header form
    Arg arglst[32];
    XmString tcs;
    XtCallbackRec callBck[5];    // Our callbacks
    
    // Setup the file selection widget - do this only once
    fileSel_widget = (Widget)XmCreateFileSelectionDialog(main_window_widget,
							 "Get file",NULL,0);
    tcs = XmStringCreateLocalized("*.*");
    XtSetArg(arglst[0], XmNx, 10);
    XtSetArg(arglst[1], XmNy, 10 );
    XtSetArg(arglst[2], XmNdirMask, tcs);  
    XtSetArg(arglst[3], XmNdefaultPosition, FALSE); 
    XtSetArg(arglst[4], XmNfontList, fixpifFontList); 
    XtSetArg(arglst[5], XmNlabelFontList,  fixpifFontList);
    XtSetArg(arglst[6], XmNbuttonFontList,  fixpifFontList);
    XtSetValues(fileSel_widget, arglst, 7);
    
    // Add callbacks to file selection widget
    callBck[0].callback= (XtCallbackProc)gotFilename;
    callBck[0].closure= 0;
    callBck[1].callback= 0;
    callBck[1].closure= 0;
    XtAddCallbacks(fileSel_widget, XmNokCallback, callBck);  
    callBck[0].callback= (XtCallbackProc)cancel_fileSel;
    callBck[0].closure= 0;
    callBck[1].callback= 0;
    callBck[1].closure= 0;
    XtAddCallbacks(fileSel_widget, XmNcancelCallback, callBck);  
    XFlush(dsply);
    XmStringFree(tcs);
    
    imageForm = widget;
    
    totX= 0;
    for(i=0; i<MAXIMGFIELDS; i++) { // Do all fields
      if(i < 23)
	XtSetArg(arglist[1], XmNy, (10 + i*30));
      else { // DO next column
	if(i < 47) { // second row
	  totX = 380;
	  XtSetArg(arglist[1], XmNy, (10 + ((i-22)*30)));
	}
	else { // third row
	  totX = 650;
	  XtSetArg(arglist[1], XmNy, (10 + ((i-46)*30)));
	}
      }
      XtSetArg(arglist[0], XmNx, (totX + 130) );
      XtSetArg(arglist[2], XmNcolumns, 20);
      XtSetArg(arglist[3], XmNfontList, font_list);      
      XtSetArg(arglist[4], XmNeditable, FALSE);      
      sprintf(numStr,"Number%d",i);
      
      imgHeadTextW[i] = XmCreateText(widget, numStr, arglist, 3);
      XtAddCallback(imgHeadTextW[i], XmNvalueChangedCallback, 
		    (XtCallbackProc) text_changed, NULL);
      XtManageChild(imgHeadTextW[i]);
      
      if(i < 23)
	XtSetArg(arglist[1], XmNy, (10 + i*30));
      else { // DO next column
	if(i < 47) { // second row
	  totX = 380;
	  XtSetArg(arglist[1], XmNy, (10 + ((i-22)*30)));
	}
	else { // third row
	  totX = 700;
	  XtSetArg(arglist[1], XmNy, (10 + ((i-46)*30)));
	}
      }
      XtSetArg(arglist[0], XmNx, (totX + 5) );
      XtSetArg(arglist[2], XmNalignment, XmALIGNMENT_BEGINNING);
      XtSetArg(arglist[3], XmNlabelString, 
	       XmStringLtoRCreate(imgFields[i], cs));
      
      XtManageChild(XmCreateLabel(widget, "", arglist, 4));
      
    }
  }
  
} // end function text_create



int askThem(char *infoStr, int dialogType)
{
  // function name: askThem
  // This is the generic display a message and get them to respond
  // screen. We get a return value from globAsk which is set by
  // infoOK() or infoCancel() (see below).

  XmString simpleString;

  // Init and try to position the window
  globAsk = -99;
  XFlush(dsply);
  simpleString = XmStringCreateLocalized(infoStr);
  XtSetArg(arglst[0], XmNmessageString, simpleString);
  XtSetArg(arglst[1], XmNdialogType, dialogType);
  XtSetArg(arglst[2], XmNx, 150);
  XtSetArg(arglst[3], XmNy, 150);
  XtSetValues(infoBox, arglst, 4);
  XFlush(dsply);
  XmStringFree(simpleString); 
 
  // Init it and pop it up sit here and wait until we get 
  // something back
  gFinished= FALSE;
  XtManageChild(infoBox);
  XtPopup(XtParent(infoBox), XtGrabNone);

  XmProcessTraversal(infoBox, XmTRAVERSE_CURRENT); 
  XtRealizeWidget(infoBox);  
  XFlush(dsply);

  // gFinished is set in the infoOK() or infoCancel() callback
  while(!gFinished)
    XtAppProcessEvent(app_context, XtIMAll);

  // Pop it down and return their response
  XtUnmanageChild(infoBox);
  XtPopdown(XtParent(infoBox));
  XtSetArg(arglst[0], XmNdialogStyle, XmDIALOG_MODELESS);
  XtSetValues(XtParent(infoBox), arglst, 1);
  XFlush(dsply);

  return(globAsk);

} // end function askThem



void infoOK(Widget widget, int *tag, XmAnyCallbackStruct *callback_data)
{
  // function name: infoOK
  // If the OK button is pressed from the infoscreen return here. Set
  // gFinished to cause the value of globAsk be return to the starting
  // point (see askThem() above).

  globAsk = TRUE;
  XtUnmanageChild(infoBox);
  gFinished= TRUE;

} // end function infoOK



void infoCancel(Widget widget, int *tag,
		XmAnyCallbackStruct *callback_data)
{
  // function name: infoCancel
  // If the CANCEL button is pressed from the infoscreen return here.
  // Set gFinished to cause the value of globAsk be return to the
  // starting point (see askThem() above).

  globAsk = FALSE;
  XtUnmanageChild(infoBox);
  gFinished= TRUE;

} // end function infoCancel



void read_activate(Widget  widget, int *tag,
		   XmAnyCallbackStruct *callback_data)
{
  // function name: read_activate
  // They selected the "Read" submenu. Branch on the appropriate.

  raiseIt(fileSel_widget); 

} // end function read_activate



void write_all_activate(Widget widget, int *tag,
			XmAnyCallbackStruct *callback_data)
{
  // function name: write_all_activate

  int i, saveCurrentHeader;

  saveCurrentHeader = currentHeader;

  for(i=0; i<=numImages; i++) {
    currentHeader = i;
    sprintf(outStr,"%d",currentHeader);
    XmTextSetString(headNumTextW, outStr);
    
    if(currentHeader==0)
      stuffGlobal();
    else 
      stuffImage(currentHeader);
    
    if(currentHeader > 0) { // image
      pifHeader.imgNum = currentHeader;
      getImage(currentHeader);
      pif_write_dh(&pifHeader);
    }
    else {
      getGlobal(currentHeader);
      pif_write_gh(&pifHeader);
    }
  }
  
  currentHeader = saveCurrentHeader;
  sprintf(outStr,"%d",currentHeader);
  XmTextSetString(headNumTextW, outStr);
  
  if(currentHeader==0)
    stuffGlobal();
  else 
    stuffImage(currentHeader);
  
  dirty = FALSE;
  
} // end function write_all_activate



void write_activate(Widget widget, int *tag,
		    XmAnyCallbackStruct *callback_data)
{
  // function name: write_activate

  if(currentHeader > 0) { // image
    pifHeader.imgNum = currentHeader;
    getImage(currentHeader);
    pif_write_dh(&pifHeader);
  }
  else { // global
    getGlobal(currentHeader);
    pif_write_gh(&pifHeader);
  }

  dirty = FALSE;

} // end function write_activate



void exit_activate(Widget widget, int *tag,
		   XmAnyCallbackStruct *callback_data)
{
  // function name: exit_activate
  // User wants to exit the program - ask again to make sure.
  
  exit(0);
  
} // end function exit_activate



void headNumArrow_activate(Widget widget, int *tag,
			   XmAnyCallbackStruct *callback_data)
{
  // function name: headNumArrow_activate
  // They want to expand the histogram on the Alter Colormap screen.

  char *thisStr;

  if(fileIsOpen) {
    
    if(dirty) { // Check to see if we need to write first
      if(askThem("Do you want to save your changes to this header?",
		 XmDIALOG_QUESTION))
	write_activate(widget, tag, callback_data);
    }
    
    thisStr = XmTextGetString(headNumTextW);
    currentHeader = (int)strtod(thisStr,0);
    XtFree(thisStr);
    
    if(*tag==XmARROW_UP)
      currentHeader++;
    else if(*tag==XmARROW_DOWN)
      currentHeader--;
    
    // Clip it accordingly
    currentHeader = MIN(currentHeader, numImages);
    currentHeader = MAX(currentHeader, 0);

    sprintf(outStr,"%d",currentHeader);
    XmTextSetString(headNumTextW, outStr);
    
    if(currentHeader == 0)
      stuffGlobal();
    else 
      stuffImage(currentHeader);
  }
  else
    askThem("You must open a file first", XmDIALOG_INFORMATION);
  
} // end function headNumArrow_activate



void TextSetInt(Widget w, int i)
{
  // function name: TextSetInt
  // Set text string based in integer input and optionally flush
  // display after setting string. Do nothing if useGUI not set.

  char str[128];

  sprintf(str, "%d", i);
  XmTextSetString(w, str);

} // end function TextSetInt



void TextSetFlt(Widget w, float f)
{
  // function name: TextSetFlt
  // Set text string based in integer input and optionally flush
  // display after setting string. Do nothing if useGUI not set.

  char str[128];

  sprintf(str, "%f", f);
  XmTextSetString(w, str);

} // end function TextSetFlt
