/* File fixpif.h */
/*******************************************************************
 * FIXPIF - 
 *
 * Include file for fixpif.c that includes global definitions
 * MACROS and forward declarations for fixpif.
 *
 * Rob Ashmore
 * Biology Department
 * Purdue University
 * Sept 1998
 *
 *******************************************************************/

#include <string.h>
#include <stdio.h>                              /* For printf and so on. */
#include <math.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>

#include <Xm/Xm.h>                      /*  */
#include <Xm/GadgetP.h>                 /* For XmGadetClass stuff */
#include <Xm/PrimitiveP.h>
#include <Xm/MessageB.h>                /* For message box stuff */
#include <Xm/ScrollBar.h>
#include <Xm/ToggleB.h>
#include <Xm/Label.h>
#include <Xm/DialogS.h>
#include <Xm/Text.h>
#include <Xm/List.h>
#include <Mrm/MrmAppl.h>                /* Motif Toolkit and MRM */

#include <X11/cursorfont.h>                /* For watch cursor */
#include <Xm/FileSB.h>

#include "../include/libPIF.h"

#define MAXGLBFIELDS 15
#define MAXIMGFIELDS 59
#define GLOBAL		 1
#define IMAGE		 2
#define HEADNUM		 3


// RAW Xlib definitions

Arg arglst[32];

XmFontList fixpifFontList;

/* Global display and applications context pointers*/
Display         *dsply; 		/* Display variable */
XtAppContext    app_context;		/* application context */

static Widget toplevel_widget, main_window_widget;

static Widget  glbHeadTextW[200], imgHeadTextW[200];
Widget fileSel_widget, fileDia_widget;
Widget imageForm, globalForm, headNumTextW, infoBox;

/* The UIL file hierarchy */
static MrmHierarchy s_MrmHierarchy;
static MrmType dummy_class;

static char *db_filename_vec[1];

static int db_filename_num =
  (sizeof db_filename_vec / sizeof db_filename_vec [0]);

XSetWindowAttributes attrs;

/***************************************
 * Generic C language work areas       *
 ***************************************/
fString_t fileName;                 /* What file do they want? */

forPifHead_t pifHeader;
char outStr[1024];

FILE *fIn;
int numCols, numRows;

int globAsk;
int gFinished;

int currentHeader;
int numImages;
int fileIsOpen;
int dirty;

/* Field labels */
String glbFields[] =
    { "Num Images",
      "ScaleF",
      "Recip ScaleF",
      "Htype",
      "Cols",
      "Rows",
      "Secs",
      "Mode",
      "EndianNess",
      "Title",
      "Even",
      "mrcX",
      "mrcY",
      "mrcZ",
      "int4scaleF",
	NULL
	};
String imgFields[] =
{ "Image Num",
    "pifMode",
    "Background",
    "i4Min",
    "i4Max",
    "r4Min",
    "r4Max",
    "Avg",
    "Std",
    "Cols",
    "Rows",
    "Secs",
    "Description",
    "aoverb",
    "map_abang",
    "a_cell",
    "b_cell",
    "c_cell",
    "alp",
    "bet",
    "gam",
    "dela",
    "delb",
    "delc",
    "a_star",
    "b_star",
    "c_star",
    "alp_star",
    "bet_star",
    "gam_star",
    "pixelSize",
    "t_matrix[0]",
    "t_matrix[1]",
    "t_matrix[2]",
    "t_matrix[3]",
    "t_matrix[4]",
    "t_matrix[5]",
    "PackRadius",
    "binFactor",
    "dthe",
    "dphi_90",
    "symmetry",
    "pixSizUnits",
    "res_out",
    "ctfMode",
    "tempFactor",
    "wiener",
    "borderType",
    "borderWidth",
    "rad_lo",
    "rad_hi",
    "transX",
    "transY",
    "transZ",
    "isTrunc",
    "origX",
    "origY",
    "origZ",
    "pifVersion",
    NULL
};




/********************************************************/
/*  FORWARD DECLARATIONS                                */
/********************************************************/
void XlibInitialize();
void s_error(char *problem_string);

void write_activate();
void write_all_activate();
void read_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void exit_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
int askThem(char *infoStr, int dialogType);
void infoOK(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void infoCancel(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void text_create();

void text_changed(Widget widget, int *tag,
		  XmAnyCallbackStruct *callback_data);

void headNumArrow_activate();
void imgNumActivate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);

void TextSetInt(Widget w, int i);
void TextSetFlt(Widget w, float f);


/* The names and addresses of things that Mrm.has to bind.  The names do
 * not have to be in alphabetical order.  */


static MrmRegisterArg reglist[] = {
    {"read_activate", (caddr_t) read_activate},
    {"write_all_activate", (caddr_t) write_all_activate},
    {"write_activate", (caddr_t) write_activate},
    {"text_create", (caddr_t) text_create},
    {"headNumArrow_activate", (caddr_t) headNumArrow_activate},
    {"infoCancel", (caddr_t) infoCancel},
    {"infoOK", (caddr_t) infoOK},
    {"exit_activate", (caddr_t) exit_activate}
};

static int reglist_num = (sizeof reglist / sizeof reglist [0]);



/* Translate keys for TEXT widgets */
char rwa_translation_table[] = 
"<Key>Return : imgNumActivate()";

XtActionsRec rwa_action_table[] = 
    {
      {"imgNumActivate",     (XtActionProc)imgNumActivate},
    };
